/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: DNAFreq.h,v 1.48 2004/07/29 00:04:44 rosada Exp $
 **/
#ifndef __DNA_FREQ_H__
#define __DNA_FREQ_H__

#include "Misc.h"

enum FreqType { FT_Naive, FT_Bg, FT_Laplace, FT_Skip};
typedef enum FreqType FreqType;

enum EntropyType { ET_Count, ET_Freq, ET_Skip };
typedef enum EntropyType EntropyType;

enum ScoreType {
  ST_Freq      , ST_FreqIc      ,
  ST_LogOdds   , ST_LogOddsIc   ,
  ST_BergHippel, ST_BergHippelIc,
  ST_Consensus , ST_ConsensusIc ,
  ST_Skip
};
typedef enum ScoreType ScoreType;

struct DNAFreq;
typedef DNAFreq *DNAFreqPtr;
typedef vector< DNAFreq > DNAFreqs;

struct DNAFreq {
  int    count[4];  // counts
  int    total;     // counts total
  double freq [4];  // frequencies
  double ic;        // information content
  double score[4];  // scores

  DNAFreq ()          { clear();   }
  DNAFreq (double gc) { setGC(gc); }
  void clear ();

  void   setGC (double gc);
  double getAT () { return freq[0] + freq[1]; }
  double getGC () { return freq[2] + freq[3]; }

  char randBase ();

  void add (int m, int w=1) { count[m]+=w; total+=w; }
  void add (DNAFreq &a);
  
  void updateFreq    (FreqType    ft, DNAFreqPtr bg);
  void updateEntropy (EntropyType et);
  void updateScore   (ScoreType   st, DNAFreqPtr bg);

  void expectedScore (DNAFreq &r, double &mu, double &var);

  void print (FILE *file=stdout, cstring prefix="", cstring suffix="\n");
};

#endif
