/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: DNAPair.h,v 1.33 2004/07/29 00:05:07 rosada Exp $
 **/
#ifndef __DNA_PAIR_H__
#define __DNA_PAIR_H__

#include "DNAFreq.h"
#include "Table.h"

struct DNAPair;
typedef DNAPair *DNAPairPtr;
typedef vector< DNAPair > DNAPairs;
typedef Table < DNAPair > DNAPairTable;

struct DNAPair {
  int     count[4][4];  // counts
  int     total;        // counts total
  double  freq [4][4];  // frequencies
  double  ic;           // information content
  double  mi;           // mutual information
  double  score[4][4];  // scores
  
  DNAPair ()                         { clear();       }
  DNAPair (DNAFreq &fi, DNAFreq &fj) { setInd(fi,fj); }
  void clear ();

  void setInd (DNAFreq &fi, DNAFreq &fj);

  void add (int m, int n, int w=1) { count[m][n]+=w; total+=w; }
  void add (DNAPair &a);

  void updateFreq    (FreqType    ft, DNAPairPtr bg);
  void updateEntropy (EntropyType et);
  void updateScore   (ScoreType   st, DNAPairPtr bg);

  void expectedScore (DNAFreq &r, double &mu, double &var);

  void print (FILE *file=stdout, cstring prefix="", cstring suffix="\n");
};

#endif
