/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: Misc.cpp,v 1.25 2004/08/18 14:19:24 rosada Exp $
 **/
#include <stdarg.h>
#include "Misc.h"

cstring strNew (int len) {
  cstring s = (cstring)malloc(len + 1);
  if (!s)
    error1("unable to allocate string of length %d", len);
  return s;
}

void strResize (cstring &s, int len) {
  s = (cstring)realloc(s, len + 1);
  if (!s)
    error1("unable to allocate string of length %d", len);
}

void strFree (cstring &s) {
  free(s);
  s = 0;
}

void doubleStr (cstring buf, double x) {
  sprintf(buf, "%f", x);
  cstring d = strchr(buf, '.');
  assert(d);

  cstring z = d + strlen(d) - 1;
  while ((*z == '0' || *z == '.') && z >= d)
    *z-- = 0;
}
cstring doubleStr (double x) {
  static char bufs[32][100];
  static int i = 0;

  cstring buf = bufs[i++];
  i %= 32;
  doubleStr(buf, x);
  return buf;
}

void _error (cstring file, int line, cstring msg) {
  fprintf(stderr, "[%s:%d] error: %s\n", file, line, msg);
  exit(EXIT_FAILURE);
}

void _errorva (cstring file, int line, cstring format, ...) {
  fprintf(stderr, "[%s:%d] error: ", file, line);

  va_list ap;
  va_start(ap, format);
  vfprintf(stderr, format, ap);
  va_end(ap);

  fprintf(stderr, "\n");
  exit(EXIT_FAILURE);
}

void _warning (cstring file, int line, cstring msg) {
  fprintf(stderr, "[%s:%d] warning: %s\n", file, line, msg);
}

void _warningva (cstring file, int line, cstring format, ...) {
  fprintf(stderr, "[%s:%d] warning: ", file, line);

  va_list ap;
  va_start(ap, format);
  vfprintf(stderr, format, ap);
  va_end(ap);

  fprintf(stderr, "\n");
}

void _message (cstring file, int line, cstring msg) {
  fprintf(stderr, "[%s:%d] %s\n", file, line, msg);
}

void _messageva (cstring file, int line, cstring format, ...) {
  fprintf(stderr, "[%s:%d] ", file, line);

  va_list ap;
  va_start(ap, format);
  vfprintf(stderr, format, ap);
  va_end(ap);

  fprintf(stderr, "\n");
}

double linPred (double x1, double y1, double x2, double y2, double x) {
  if (x1 == x2)
    error("equal x's");
  double t = (x2 - x) / (x2 - x1);
  return y1*t + y2*(1-t);
}

int readLine (FILE *file, cstring buf, int maxlen) {
  if (fgets(buf, maxlen, file) == NULL)
    return 0;

  // trim newline(s)
  int len = strlen(buf);
  if (buf[len-1] == '\n') buf[--len] = 0;
  if (buf[len-1] == '\r') buf[--len] = 0;
  return len;
}

int fpeek (FILE *file) {
  int c = fgetc(file);
  ungetc(c, file);
  return c;
}

