/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: Misc.h,v 1.43 2004/07/29 19:33:21 rosada Exp $
 **/
#ifndef __MISC_H__
#define __MISC_H__

#include "Config.h"
#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <vector>
using namespace std;


/**
 * basic data types.
 **/
typedef       void  *voidPtr;
typedef const void *cvoidPtr;

typedef char *cstring;
cstring strNew    (            int len);
void    strResize (cstring &s, int len);
void    strFree   (cstring &s         );

typedef vector< bool > bools;

inline int boolCmp  (bool a, bool b) { return a-b; }
inline int boolRCmp (bool a, bool b) { return b-a; }

typedef vector< double > doubles;

inline int doubleCmp  (double a, double b) { return (a<b) ? -1 : (a>b) ?  1 : 0; }
inline int doubleRCmp (double a, double b) { return (a<b) ?  1 : (a>b) ? -1 : 0; }

inline int _doubleCmp  (cvoidPtr a, cvoidPtr b) { assert(a && b); return doubleCmp (*(double *)a, *(double *)b); }
inline int _doubleRCmp (cvoidPtr a, cvoidPtr b) { assert(a && b); return doubleRCmp(*(double *)a, *(double *)b); }

void    doubleStr (cstring buf, double x);
cstring doubleStr (double x);

typedef vector< int > ints;

inline int intCmp  (int a, int b) { return (a<b) ? -1 : (a>b) ?  1 : 0; }
inline int intRCmp (int a, int b) { return (a<b) ?  1 : (a>b) ? -1 : 0; }

inline int _intCmp  (cvoidPtr a, cvoidPtr b) { assert(a && b); return intCmp (*(int *)a, *(int *)b); }
inline int _intRCmp (cvoidPtr a, cvoidPtr b) { assert(a && b); return intRCmp(*(int *)a, *(int *)b); }


/**
 * error, warning and message.
 **/
#define error(f)          _error  (__FILE__,__LINE__,f)
#define error1(f,a)       _errorva(__FILE__,__LINE__,f,a)
#define error2(f,a,b)     _errorva(__FILE__,__LINE__,f,a,b)
#define error3(f,a,b,c)   _errorva(__FILE__,__LINE__,f,a,b,c)

void _error   (cstring file, int line, cstring msg);
void _errorva (cstring file, int line, cstring format, ...);

#define warning(f)        _warning  (__FILE__,__LINE__,f)
#define warning1(f,a)     _warningva(__FILE__,__LINE__,f,a)
#define warning2(f,a,b)   _warningva(__FILE__,__LINE__,f,a,b)
#define warning3(f,a,b,c) _warningva(__FILE__,__LINE__,f,a,b,c)

void _warning   (cstring file, int line, cstring msg);
void _warningva (cstring file, int line, cstring format, ...);

#define message(f)        _message  (__FILE__,__LINE__,f)
#define message1(f,a)     _messageva(__FILE__,__LINE__,f,a)
#define message2(f,a,b)   _messageva(__FILE__,__LINE__,f,a,b)
#define message3(f,a,b,c) _messageva(__FILE__,__LINE__,f,a,b,c)

void _message   (cstring file, int line, cstring msg);
void _messageva (cstring file, int line, cstring format, ...);


/**
 * math.
 **/
inline int min (int a, int b)               { return a<b ? a : b; }
inline int min (int a, int b, int c)        { return min(a,min(b,c)); }
inline int min (int a, int b, int c, int d) { return min(a,min(b,c,d)); }
inline int max (int a, int b)               { return a>b ? a : b; }
inline int max (int a, int b, int c)        { return max(a,max(b,c)); }
inline int max (int a, int b, int c, int d) { return max(a,max(b,c,d)); }
inline int mod (int x, int y) { x %= y; if (x < 0) x += y; return x; }

inline double min (double a, double b)                     { return a<b ? a : b; }
inline double min (double a, double b, double c)           { return min(a,min(b,c)); }
inline double min (double a, double b, double c, double d) { return min(a,min(b,c,d)); }
inline double max (double a, double b)                     { return a>b ? a : b; }
inline double max (double a, double b, double c)           { return max(a,max(b,c)); }
inline double max (double a, double b, double c, double d) { return max(a,max(b,c,d)); }

inline bool isEven (int a) { return !(a % 2); }
inline bool isOdd  (int a) { return a % 2; }

inline int firstNonZero (int a, int b)               { return a ? a : b; }
inline int firstNonZero (int a, int b, int c)        { return a ? a : firstNonZero(b,c); }
inline int firstNonZero (int a, int b, int c, int d) { return a ? a : firstNonZero(b,c,d); }

double linPred (double x1, double y1, double x2, double y2, double x);


/**
 * random.
 **/
inline void   randInit () { srandom((unsigned)time(0)); }
inline double randUnit () { return (double)random() / ((unsigned)1 << 31); }  // [0,1)


/**
 * vector functions.
 **/
template <class T> void deleteVec (vector< T* > &v) {
  for (int i=0; i<v.size(); i++) {
    delete v[i];
    v[i] = 0;
  }
  v.clear();
}

template <class T> void pushBack (vector< T > &a, vector< T > &b) {
  assert(&a != &b);
  a.reserve(a.size() + b.size());
  for (int i=0; i<b.size(); i++)
    a.push_back(b[i]);
}


/**
 * file io.
 **/
int readLine (FILE *file, cstring buf, int maxlen);
int fpeek    (FILE *file);

#endif
