/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: Site.h,v 1.38 2004/07/29 15:41:29 rosada Exp $
 **/
#ifndef __SITE_H__
#define __SITE_H__

#include <float.h>
#include <queue>
#include "Misc.h"


/**
 * Site.
 **/
struct Site {
  cstring desc;
  cstring seq;
  int     loc;
  bool    pStrand;
  int     len;
  double  score;
  double  pvalue;
  int     rank;
  double  fp;
  
  Site (cstring desc, cstring seq, int loc, bool pStrand, int len, double score, double pvalue=-1, int rank=-1, double fp=-1);
  Site (Site &s);
 ~Site ();

  bool isRComp ();

  void print (FILE *file=stdout, cstring prefix="", cstring suffix="\n");
};
typedef         Site      *SitePtr;
typedef vector< SitePtr >  Sites;



/**
 * sort by score (increasing/decreasing),
 * then location,
 * then strand (positive strand first).
 **/
int siteCmp  (SitePtr a, SitePtr b);
int siteRCmp (SitePtr a, SitePtr b);

inline int _siteCmp  (cvoidPtr a, cvoidPtr b) { return siteCmp (*(SitePtr *)a, *(SitePtr *)b); }
inline int _siteRCmp (cvoidPtr a, cvoidPtr b) { return siteRCmp(*(SitePtr *)a, *(SitePtr *)b); }


/**
 * sort by description, then by above.
 **/
int siteDescCmp  (SitePtr a, SitePtr b);
int siteDescRCmp (SitePtr a, SitePtr b);

inline int _siteDescCmp  (cvoidPtr a, cvoidPtr b) { return siteDescCmp (*(SitePtr *)a, *(SitePtr *)b); }
inline int _siteDescRCmp (cvoidPtr a, cvoidPtr b) { return siteDescRCmp(*(SitePtr *)a, *(SitePtr *)b); }


/**
 * Data structure useful for finding the highest scoring site.
 * Does not dynamically allocate any memory, so copy at will.
 **/
struct BestSite {
  cstring desc;
  cstring seq;
  int     loc;
  bool    pStrand;
  int     len;
  double  score;
  int     numTies;
  double  pvalue;
  int     rank;
  double  fp;

  BestSite () : score(-DBL_MAX) {}

  void operator   () (cstring desc, cstring seq, int loc, bool pStrand, int len, double score, double pvalue=-1, int rank=-1, double fp=-1);
  SitePtr newSite () { return new Site(desc, seq, loc, pStrand, len, score, pvalue, rank, fp); }
};    


/**
 * Data structure useful for finding the highest N scoring sites.
 **/
struct TopSitesCompare {
  bool operator () (SitePtr a, SitePtr b) { return b->score < a->score; }
};
struct TopSites {
  int n;
  priority_queue< SitePtr, Sites, TopSitesCompare > sites;
  
  TopSites (int n_) : n(n_) {}
  void operator () (BestSite &bs);
  int  getAll      (Sites &sites);
};

#endif
