/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: SitePairScorer.h,v 1.7 2004/06/02 17:03:25 rosada Exp $
 *
 * Score a site using pairs of columns.
 **/
#ifndef __SITE_PAIR_SCORER_H__
#define __SITE_PAIR_SCORER_H__

#include "DNAPair.h"
#include "SiteScorer.h"

enum BgType { BT_Ind, BT_All, BT_Scope };
typedef enum BgType BgType;

struct SitePairScorer : public SiteScorer {
  BgType    bt;
  ScoreType pst;
  int       scope;
  DNAPair   obg2;

  DNAPair      bg2;  // global background pair
  DNAPairs     bgS;  // background for each scope
  DNAPairTable pairs;

  SitePairScorer (
    FreqType ft, ScoreType  st, int len  , DNAFreq &obg,
    BgType   bt, ScoreType pst, int scope
  );
  void clear ();

  void add (cstring seq, bool pStrand=true, int weight=1);

  void update ();
  void update (DNAPair &p, DNAPair &bg);

  double operator() (cstring seq, bool pStrand);

  void expectedScore (DNAFreq &r, double &mu, double &var);
};

struct PairProfileRigorous : public SitePairScorer {
  PairProfileRigorous (int len, DNAFreq obg, BgType bt);
  void update ();
};

struct PairCentroidScorer : public SitePairScorer {
  PairCentroidScorer (int len, DNAFreq obg, int scope);
  double operator() (cstring seq, bool pStrand);
};

#endif
