/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: SiteScorer.cpp,v 1.10 2004/06/10 17:42:19 rosada Exp $
 **/
#include "DNA.h"
#include "SiteScorer.h"

SiteScorer::SiteScorer (FreqType ft_, ScoreType st_, int len_, DNAFreq &obg_) :
  ft(ft_), st(st_), len(len_), len1(len-1), obg(obg_),
  size(0), col(len), totalIc(0)
{
  if (len <= 0)
    error("length out of range");
}

void SiteScorer::clear () {
  size = 0;
  bg.clear();
  for (int i=0; i<len; i++)
    col[i].clear();
  totalIc = 0;
}

void SiteScorer::add (cstring seq, bool pStrand, int w) {
  size += w;
  if (pStrand) {
    cstring s = seq;
    for (int i=0; i<len; i++) {
      int m = baseIndex(*s++);
      bg.add(m,w);
      col[i].add(m,w);
    }
  }
  else {
    cstring s = seq + len1;
    for (int i=0; i<len; i++) {
      int m = compIndex(*s--);
      bg.add(m,w);
      col[i].add(m,w);
    }
  }
}
void SiteScorer::add (SitePtr s, int w) {
  assert(s);
  add(s->seq, true, w);
}
void SiteScorer::add (Sites &sites, int w) {
  for (int i=0; i<sites.size(); i++)
    add(sites[i], w);
}

void SiteScorer::update () {
  update(bg, obg);
  totalIc = 0;
  for (int i=0; i<len; i++) {
    DNAFreq &c = col[i];
    update(c, bg);
    totalIc += c.ic;
  }
}
void SiteScorer::update (DNAFreq &f, DNAFreq &bg) {
  f.updateFreq   (ft, &bg );
  f.updateEntropy(ET_Count);
  f.updateScore  (st, &bg );
}

double SiteScorer::operator() (cstring seq, bool pStrand) {
  double score = 0;
  if (pStrand) {
    cstring s = seq;
    for (int i=0; i<len; i++) {
      int m = baseIndex(*s++);
      score += col[i].score[m];
    }
  }
  else {
    cstring s = seq + len1;
    for (int i=0; i<len; i++) {
      int m = compIndex(*s--);
      score += col[i].score[m];
    }
  }
  return score;
}

void SiteScorer::expectedScore (DNAFreq &r, double &mu, double &var) {
  for (int i=0; i<len; i++)
    col[i].expectedScore(r, mu, var);
}


/**
 * Centroid.
 * Eliminates rounding errors.
 **/
CentroidScorer::CentroidScorer (int len, DNAFreq &obg) :
  SiteScorer(FT_Naive, ST_Freq, len, obg)
{}

double CentroidScorer::operator() (cstring seq, bool pStrand) {
  double score = SiteScorer::operator()(seq, pStrand);
  return round(score * size) / size;
}
