/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: SiteScorerFactory.cpp,v 1.8 2004/06/22 03:00:32 rosada Exp $
 **/
#include "SitePairScorer.h"
#include "SiteScorerFactory.h"

SiteScorerPtr makeSingleton (cstring m,        bool ic, cstring var, int len, DNAFreq &obg);
SiteScorerPtr makePair      (cstring m, int s, bool ic, cstring var, int len, DNAFreq &obg);

SiteScorerPtr makeSiteScorer (cstring method, cstring scope, bool ic, cstring var, int len, DNAFreq &obg) {
  int s;
  if (!strcasecmp(scope, "all") || !strcasecmp(scope, "full"))
    s = len - 1;
  else if (!strcasecmp(scope, "half"))
    s = len / 2;
  else if (1==sscanf(scope, "%d", &s))
    ;
  else 
    error1("unknown scope %s\n", scope);
  
  if (!s)
    return makeSingleton(method, ic, var, len, obg);
  else
    return makePair(method, s, ic, var, len, obg);
}

SiteScorerPtr makeSingleton (cstring m, bool ic, cstring var, int len, DNAFreq &obg) {
  if (!strcasecmp(m, "centroid")) {
    if (var) error("unknown variant");
    if (!ic) return new CentroidScorer(len, obg);
    else     return new SiteScorer(FT_Naive, ST_FreqIc, len, obg);
  }
  if (!strcasecmp(m, "pssm") || !strcasecmp(m, "profile")) {
    if (var) error("unknown variant");
    if (!ic) return new SiteScorer(FT_Bg, ST_LogOdds  , len, obg);
    else     return new SiteScorer(FT_Bg, ST_LogOddsIc, len, obg);
  }
  if (!strcasecmp(m, "bergHippel")) {
    if (var) error("unknown variant");
    if (!ic) return new SiteScorer(FT_Skip, ST_BergHippel  , len, obg);
    else     return new SiteScorer(FT_Skip, ST_BergHippelIc, len, obg);
  }
  if (!strcasecmp(m, "consensus")) {
    if (var) error("unknown variant");
    if (!ic) return new SiteScorer(FT_Naive, ST_Consensus  , len, obg);
    else     return new SiteScorer(FT_Naive, ST_ConsensusIc, len, obg);
  }
  error1("unknown site scoring method \'%s\'\n", m);
  return 0;
}

SiteScorerPtr makePair (cstring m, int s, bool ic, cstring var, int len, DNAFreq &obg) {
  if (!strcasecmp(m, "centroid")) {
    if (var) error("unknown variant");
    if (!ic) return new PairCentroidScorer(len, obg, s);
    else     return new SitePairScorer(FT_Naive, ST_FreqIc, len, obg, BT_Ind, ST_FreqIc, s);
  }
  if (!strcasecmp(m, "pssm") || !strcasecmp(m, "profile")) {
    if (!var)
      var = "ind";
    if (!ic) {
           if (!strcasecmp(var, "ind"  )) { return new SitePairScorer(FT_Bg, ST_LogOdds, len, obg, BT_Ind  , ST_LogOdds, s); } // independent background
      else if (!strcasecmp(var, "all"  )) { return new SitePairScorer(FT_Bg, ST_LogOdds, len, obg, BT_All  , ST_LogOdds, s); } // all pairs background
      else if (!strcasecmp(var, "rig"  )) { if (s != 1) error("Rigorous pair profile requires scope 1");
	                                    return new PairProfileRigorous(len, obg, BT_All);                                } // rigorous derivation
      else if (!strcasecmp(var, "scope")) { return new SitePairScorer(FT_Bg, ST_LogOdds, len, obg, BT_Scope, ST_LogOdds, s); } // scope background
      else error("unknown variant");
    }
    else {
           if (!strcasecmp(var, "ind"  )) { return new SitePairScorer(FT_Bg, ST_LogOddsIc, len, obg, BT_Ind  , ST_LogOddsIc, s); } // independent background
      else if (!strcasecmp(var, "all"  )) { return new SitePairScorer(FT_Bg, ST_LogOddsIc, len, obg, BT_All  , ST_LogOddsIc, s); } // joint background
      else if (!strcasecmp(var, "rig"  )) { if (s != 1) error("Rigorous pair profile requires scope 1");
 	                                    return new PairProfileRigorous(len, obg, BT_All);                                    } // rigorous derivation
      else if (!strcasecmp(var, "scope")) { return new SitePairScorer(FT_Bg, ST_LogOddsIc, len, obg, BT_Scope, ST_LogOddsIc, s); } // scope background
      else error("unknown variant");
    }
  }
  if (!strcasecmp(m, "bergHippel")) {
    if (var) error("unknown variant");
    if (!ic) return new SitePairScorer(FT_Skip, ST_BergHippel  , len, obg, BT_Ind, ST_BergHippel  , s);
    else     return new SitePairScorer(FT_Skip, ST_BergHippelIc, len, obg, BT_Ind, ST_BergHippelIc, s);
  }
  if (!strcasecmp(m, "consensus")) {
    if (var) error("unknown variant");
    if (!ic) return new SitePairScorer(FT_Naive, ST_Consensus  , len, obg, BT_Ind, ST_Consensus  , s);
    else     return new SitePairScorer(FT_Naive, ST_ConsensusIc, len, obg, BT_Ind, ST_ConsensusIc, s);
  }
  error1("unknown site scoring method \'%s\'\n", m);
  return 0;
}
