/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: Table.h,v 1.12 2004/07/29 00:07:36 rosada Exp $
 *
 * Entries arranged in C order.
 **/
#ifndef __TABLE_H__
#define __TABLE_H__

#include "Misc.h"


/**
 * 2D table.
 **/
template <class T>
struct Table {
  int rows, cols;
  vector< T > xs;

  Table (int rs, int cs) :
    rows(rs), cols(cs), xs(rs*cs)
  { assert(rows>=0 && cols>=0); }

  Table (int s=0) :
    rows(s), cols(s), xs(s*s)
  { assert(s>=0); }

  int size () { return xs.size(); }
  
  void resize (int rs, int cs) {
    assert(rs>=0 && cs>=0);
    rows = rs;
    cols = cs;
    xs.resize(rows * cols);
  }

  T& operator () (int r, int c) {
    assert(0<=r && r<rows && 0<=c && c<cols);
    return xs[c + cols*r];
  }

  T& operator [] (int i) {
    assert(0<=i && i<size());
    return xs[i];
  }

  Table< T >& operator = (T &x) {
    for (int i=0; i<xs.size(); i++)
      xs[i] = x;
    return *this;
  }
};
typedef Table< double > doubleTable;
typedef Table< int >       intTable;


/**
 * 3D Table.
 **/
template <class T>
struct Table3 {
  int pages, rows, cols;
  vector< T > xs;

  Table3 (int ps, int rs, int cs) :
    pages(ps), rows(rs), cols(cs), xs(ps*rs*cs)
  { assert(pages>=0 && rows>=0 && cols>=0); }

  Table3 (int s=0) :
    pages(s), rows(s), cols(s), xs(s*s*s)
  { assert(s>=0); }

  int size () { return xs.size(); }

  void resize (int ps, int rs, int cs) {
    assert(ps>=0 && rs>=0 && cs>=0);
    pages = ps;
    rows  = rs;
    cols  = cs;
    xs.resize(pages * rows * cols);
  }

  T& operator () (int p, int r, int c) {
    assert(0<=p && p<pages && 0<=r && r<rows && 0<=c && c<cols);
    return xs[c + cols*(r + rows*p)];
  }

  T& operator [] (int i) {
    assert(0<=i && i<size());
    return xs[i];
  }

  Table< T >& operator = (T &x) {
    for (int i=0; i<xs.size(); i++)
      xs[i] = x;
    return *this;
  }
};
typedef Table3< double > doubleTable3;
typedef Table3< int >       intTable3;

#endif
