/**
 * This program is free software, you can redistribute it and/or
 * modify it under the terms of the GNU General Public License, a copy
 * of which is provided under the name LICENSE.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Copyright 2004 Robert Osada
 * $Id: DNA.h,v 1.11 2004/06/02 17:04:16 rosada Exp $
 *
 * With the exception of isDNA, routines will die on bad input.
 **/
#ifndef __DNA_H__
#define __DNA_H__

#include "Misc.h"


/**
 * is this DNA?
 **/
inline bool isDNA (char b) {
  switch (b) {  // approximately twice as fast as using islower
  case 'a': case 'A':
  case 't': case 'T':
  case 'g': case 'G':
  case 'c': case 'C': return true;
  }
  return false;
}


/**
 * base/index complement.
 **/
inline char comp (char b) {
  switch (b) {
  case 'a': return 't';  case 'A': return 'T';
  case 't': return 'a';  case 'T': return 'A';
  case 'g': return 'c';  case 'G': return 'C';
  case 'c': return 'g';  case 'C': return 'G';
  }
  error1("invalid base '%c'", b);
  return 0;
}

inline int comp (int i) {
  switch (i) {
  case 0: return 1;
  case 1: return 0;
  case 2: return 3;
  case 3: return 2;
  }
  error1("invalid index '%d'", i);
  return -1;
}


/** 
 * base/complement to index.
 **/
inline int baseIndex (char b) {
  switch (b) {
  case 'a': case 'A': return  0;
  case 't': case 'T': return  1;
  case 'g': case 'G': return  2;
  case 'c': case 'C': return  3;
  }
  error1("invalid base '%c'", b);
  return -1;
}

inline int compIndex (char b) {
  switch (b) {
  case 'a': case 'A': return  1;
  case 't': case 'T': return  0;
  case 'g': case 'G': return  3;
  case 'c': case 'C': return  2;
  }
  error1("invalid base '%c'", b);
  return -1;
}


/**
 * index to base.
 **/
inline char indexBase (int i) {
  switch (i) {
  case 0: return 'a';
  case 1: return 't';
  case 2: return 'g';
  case 3: return 'c';
  }
  error1("invalid index '%d'", i);
  return 0;
}


/** 
 * strings.
 **/
bool isDNA       (cstring seq, int len);
bool isRComp     (cstring seq, int len);
int  hammingDist (cstring seq, cstring seq2, bool pStrand, int len);

#endif
